@@ -1,3 +1,5 @@
+# ruff: noqa: E402
+
 from __future__ import (
     annotations,
 )  # work around Python 3.8 issue, see https://stackoverflow.com/a/68072481/335756
@@ -19,7 +21,7 @@ from __future__ import (
 # Import cysignals early so it doesn't disrupt Scalene's use of signals; this allows Scalene to profile Sage.
 # See https://github.com/plasma-umass/scalene/issues/740.
 try:
-    import cysignals
+    import cysignals # noqa: F401
 except ModuleNotFoundError:
     pass
 
@@ -56,7 +58,12 @@ from scalene.time_info import get_times, TimeInfo
 from scalene.redirect_python import redirect_python
 
 from collections import defaultdict
-from types import FrameType
+from types import (
+    BuiltinFunctionType,
+    FrameType,
+    FunctionType,
+    ModuleType,
+)
 from typing import (
     Any,
     Callable,
@@ -155,6 +162,28 @@ def enable_profiling() -> Generator[None, None, None]:
     yield
     stop()
 
+def patch_module_functions_with_signal_blocking(module: ModuleType, signal_to_block: signal.Signals) -> None:
+    """Patch all functions in the given module to block the specified signal during execution."""
+    
+    def signal_blocking_wrapper(func: Union[BuiltinFunctionType, FunctionType]) -> Any:
+        """Wrap a function to block the specified signal during its execution."""
+        @functools.wraps(func)
+        def wrapped(*args: Any, **kwargs: Any) -> Any:
+            # Block the specified signal temporarily
+            original_sigmask = signal.pthread_sigmask(signal.SIG_BLOCK, [signal_to_block])
+            try:
+                return func(*args, **kwargs)
+            finally:
+                # Restore original signal mask
+                signal.pthread_sigmask(signal.SIG_SETMASK, original_sigmask)
+        return wrapped
+
+    # Iterate through all attributes of the module
+    for attr_name in dir(module):
+        attr = getattr(module, attr_name)
+        if isinstance(attr, BuiltinFunctionType) or isinstance(attr, FunctionType):
+            wrapped_attr = signal_blocking_wrapper(attr)
+            setattr(module, attr_name, wrapped_attr)
 
 class Scalene:
     """The Scalene profiler itself."""
@@ -631,10 +660,14 @@ class Scalene:
         arguments: argparse.Namespace,
         program_being_profiled: Optional[Filename] = None,
     ) -> None:
+        # Wrap all os calls so that they disable SIGALRM (the signal used for CPU sampling).
+        # This fixes https://github.com/plasma-umass/scalene/issues/841.
+        if sys.platform != "win32":
+            patch_module_functions_with_signal_blocking(os, signal.SIGALRM)
+
+        # Import all replacement functions.
         import scalene.replacement_exit
         import scalene.replacement_get_context
-
-        # Hijack lock, poll, thread_join, fork, and exit.
         import scalene.replacement_lock
         import scalene.replacement_mp_lock
         import scalene.replacement_pjoin
@@ -643,9 +676,9 @@ class Scalene:
 
         if sys.platform != "win32":
             import scalene.replacement_fork
-            import scalene.replacement_poll_selector
+            import scalene.replacement_poll_selector # noqa: F401
 
-        Scalene.__args = cast(ScaleneArguments, arguments)
+        Scalene.__args = ScaleneArguments(**vars(arguments))
         Scalene.__alloc_sigq = ScaleneSigQueue(
             Scalene.alloc_sigqueue_processor
         )
@@ -660,12 +693,12 @@ class Scalene:
 
         Scalene.__windows_queue = queue.Queue()
         if sys.platform == "win32":
-            if arguments.memory:
+            if Scalene.__args.memory:
                 print(
                     "Scalene warning: Memory profiling is not currently supported for Windows.",
                     file=sys.stderr,
                 )
-                arguments.memory = False
+                Scalene.__args.memory = False
 
         # Initialize the malloc related files; if for whatever reason
         # the files don't exist and we are supposed to be profiling
@@ -675,19 +708,19 @@ class Scalene:
             Scalene.__memcpy_mapfile = ScaleneMapFile("memcpy")
         except Exception:
             # Ignore if we aren't profiling memory; otherwise, exit.
-            if arguments.memory:
+            if Scalene.__args.memory:
                 sys.exit(1)
 
-        Scalene.__signals.set_timer_signals(arguments.use_virtual_time)
+        Scalene.__signals.set_timer_signals(Scalene.__args.use_virtual_time)
         Scalene.__profiler_base = str(os.path.dirname(__file__))
-        if arguments.pid:
+        if Scalene.__args.pid:
             # Child process.
             # We need to use the same directory as the parent.
             # The parent always puts this directory as the first entry in the PATH.
             # Extract the alias directory from the path.
             dirname = os.environ["PATH"].split(os.pathsep)[0]
             Scalene.__python_alias_dir = pathlib.Path(dirname)
-            Scalene.__pid = arguments.pid
+            Scalene.__pid = Scalene.__args.pid
 
         else:
             # Parent process.
@@ -700,7 +733,7 @@ class Scalene:
             Scalene.__pid = 0
             cmdline = ""
             # Pass along commands from the invoking command line.
-            if "off" in arguments and arguments.off:
+            if "off" in Scalene.__args and Scalene.__args.off:
                 cmdline += " --off"
             for arg in [
                 "use_virtual_time",
@@ -713,12 +746,12 @@ class Scalene:
                 "no_browser",
                 "reduced_profile",
             ]:
-                if getattr(arguments, arg):
+                if getattr(Scalene.__args, arg):
                     cmdline += f'  --{arg.replace("_", "-")}'
             # Add the --pid field so we can propagate it to the child.
             cmdline += f" --pid={os.getpid()} ---"
             # Build the commands to pass along other arguments
-            environ = ScalenePreload.get_preload_environ(arguments)
+            environ = ScalenePreload.get_preload_environ(Scalene.__args)
             if sys.platform == "win32":
                 preface = "\n".join(
                     f"set {k}={str(v)}\n" for (k, v) in environ.items()
@@ -1890,18 +1923,18 @@ class Scalene:
     @staticmethod
     def process_args(args: argparse.Namespace) -> None:
         """Process all arguments."""
-        Scalene.__args = cast(ScaleneArguments, args)
+        Scalene.__args = ScaleneArguments(**vars(args))
         Scalene.__next_output_time = (
             time.perf_counter() + Scalene.__args.profile_interval
         )
-        Scalene.__output.html = args.html
-        if args.outfile:
+        Scalene.__output.html = Scalene.__args.html
+        if Scalene.__args.outfile:
             Scalene.__output.output_file = os.path.abspath(
-                os.path.expanduser(args.outfile)
+                os.path.expanduser(Scalene.__args.outfile)
             )
-        Scalene.__is_child = args.pid != 0
+        Scalene.__is_child = Scalene.__args.pid != 0
         # the pid of the primary profiler
-        Scalene.__parent_pid = args.pid if Scalene.__is_child else os.getpid()
+        Scalene.__parent_pid = Scalene.__args.pid if Scalene.__is_child else os.getpid()
         # Don't profile the GPU if not enabled (i.e., either no options or --cpu and/or --memory, but no --gpu).
         if not Scalene.__args.gpu:
             Scalene.__output.gpu = False
@@ -2083,7 +2116,7 @@ class Scalene:
                         # TODO: Add in entrypoint_dir logic for `-m` operation
                         Scalene.__entrypoint_dir = program_path
                     # If a program path was specified at the command-line, use it.
-                    if len(args.program_path) > 0:
+                    if len(Scalene.__args.program_path) > 0:
                         Scalene.__program_path = Filename(
                             os.path.abspath(args.program_path)
                         )
